<?php

namespace Drupal\pagedesigner_webform\Plugin\pagedesigner\Handler;

use Drupal\Core\Cache\Cache;
use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\Plugin\HandlerPluginBase;
use Drupal\ui_patterns\Definition\PatternDefinition;
use Drupal\webform\Entity\Webform as Form;

/**
 * Process entities of type "webform".
 *
 * @PagedesignerHandler(
 *   id = "webform",
 *   name = @Translation("Webform handler"),
 *   types = {
 *      "webform",
 *      "form"
 *   },
 * )
 */
class Webform extends HandlerPluginBase {

  /**
   * The definition of the webform pattern.
   *
   * @var array
   */
  protected $definition = [
    "id" => 'webform:',
    "pagedesigner" => 1,
    "icon" => "far fa-edit",
    "type" => "webform",
    "category" => 'Forms',
    "label" => '',
    "description" => '',
  ];

  /**
   * {@inheritdoc}
   */
  public function collectAttachments(array &$attachments) {
    $attachments['library'][] = 'pagedesigner_webform/pagedesigner';
  }

  /**
   * {@inheritdoc}
   */
  public function collectPatterns(array &$patterns) {
    $formIds = \Drupal::entityQuery('webform')->execute();
    foreach ($formIds as $formId) {
      if (strpos($formId, 'example_') !== 0 && strpos($formId, 'template_') !== 0) {
        $definition = $this->definition;
        $id = $definition['id'] . $formId;
        $cid = 'pagedesigner_definition_form_' . $formId;
        if ($item = \Drupal::cache()->get($cid)) {
          $patterns[$id] = new PatternDefinition($item->data);
        }
        else {
          $form = Form::load($formId);
          if ($form != NULL) {
            $definition = $this->definition;
            $definition['webform'] = $formId;
            $build = [
              $formId => [
                '#type' => 'webform',
                '#webform' => $formId,
              ],
            ];
            $markup = \Drupal::service('renderer')->renderPlain($build);
            $markup = '<div data-gjs-type="webform">' . $markup . '<div>';
            $definition['markup'] = $markup;
            $definition['label'] = (string) $form->label();
            $patterns[$id] = new PatternDefinition($definition);
            \Drupal::cache()->set($cid, $definition, Cache::PERMANENT, $form->getCacheTags());
          }
        }
      }

    }
  }

  /**
   * {@inheritdoc}
   */
  public function get(Element $entity, string &$result = '') {
  }

  /**
   * {@inheritdoc}
   */
  public function serialize(Element $entity, array &$result = []) {
    $result['webform_id'] = $entity->field_webform->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function render(Element $entity, array &$build = []) {
    if ($entity->field_webform->entity != NULL) {
      $build = [
        '#type' => 'webform',
        '#webform' => $entity->field_webform->target_id,
        '#entity' => $entity,
        '#prefix' => '<div id="pd-cp-' . $entity->id() . '">',
        '#suffix' => '</div>',
      ] + $build;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function renderForEdit(Element $entity, array &$build = []) {
    $build = $this->elementHandler->render($entity) + $build;
    $build['#prefix'] = '<div data-gjs-type="webform:' . $entity->field_webform->target_id . '" data-entity-id="' . $entity->id() . '" id="pd-cp-' . $entity->id() . '">';
    $build['#edit'] = TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function describe(Element $entity, array &$result = []) {
    parent::describe($entity, $result);
    $result['type'] = 'webform';
  }

  /**
   * {@inheritdoc}
   */
  public function generate($definition, array $data, Element &$entity = NULL) {
    $formId = $definition['additional']['webform'];
    $form = Form::load($formId);
    if ($form == NULL) {
      throw new BadRequestHttpException('The given form does not exist.');
    }
    parent::generate(['type' => 'webform', 'name' => 'webform'], $data, $entity);
    $entity->field_webform->entity = $form;
    $entity->saveEdit();

    // Adding entity to parent cell if provided.
    $cell = $entity->parent->entity;
    if ($cell != NULL) {
      $cell = $cell->getTranslation($entity->langcode->value);
      $cell->children->appendItem($entity);
      $cell->saveEdit();
    }
  }

}
