<?php

namespace Drupal\pagedesigner\Plugin;

use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\Service\Renderer;

/**
 * Povides the compound handler for structural pagedesigner elements.
 *
 * The structural handler provides methods for handling pagedesigner elements
 * like rows and components.
 * It is to be extended and the inherited class
 * to be annotated for the correct type of pagedesiger element.
 */
class CompoundHandlerBase extends HandlerPluginBase {

  /**
   * {@inheritdoc}
   */
  public function renderForPublic(Element $entity, array &$build) {
    $entity = $entity->loadNewestPublished();
    if ($entity != NULL) {
      $fields = [];
      $fieldsSimple = $this->collectFields($entity, 'renderForPublic');
      foreach ($fieldsSimple as $placeholder => $value) {
        if (is_array($value) && isset($value['#plain_text'])) {
          $fields[$placeholder] = $value['#plain_text'];
        }
        else {
          $fields[$placeholder] = $value;
        }
      }
      $pattern = $entity->field_pattern->value;
      $build = [
        '#type' => 'pattern',
        '#id' => $pattern,
        '#fields' => $fields,
        '#entity' => $entity,
      ] + $build;

      $this->addStyles($entity);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function render(Element $entity, array &$build = []) {
    $fields = [];
    $fieldsSimple = $this->collectFields($entity, 'render');
    foreach ($fieldsSimple as $placeholder => $value) {
      if (is_array($value) && isset($value['#plain_text'])) {
        $fields[$placeholder] = $value['#plain_text'];
      }
      else {
        $fields[$placeholder] = $value;
      }
    }
    $pattern = $entity->field_pattern->value;
    $build = [
      '#type' => 'pattern',
      '#id' => $pattern,
      '#fields' => $fields,
      '#entity' => $entity,
    ] + $build;

    $this->addStyles($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function renderForEdit(Element $entity, array &$build = []) {
    $fields = [];
    $fieldsSimple = $this->collectFields($entity, 'renderForEdit');
    foreach ($fieldsSimple as $placeholder => $value) {
      if (is_array($value) && isset($value['#plain_text'])) {
        $fields[$placeholder] = $value['#plain_text'];
      }
      else {
        $fields[$placeholder] = $value;
      }
    }
    $pattern = $entity->field_pattern->value;
    $build = [
      '#type' => 'pattern',
      '#id' => $pattern,
      '#fields' => $fields,
      '#entity' => $entity,
      '#edit' => TRUE,
    ] + $build;

    $this->addStyles($entity);
  }

  /**
   * Add the styles of the given entity to the renderer.
   *
   * @param \Drupal\pagedesigner\Entity\Element $entity
   *   The entity to add the styles of.
   */
  protected function addStyles(Element $entity) {
    if ($entity->hasField('field_styles')) {

      $styles = $entity->field_styles;
      foreach ($styles as $item) {
        Renderer::addStyle(
            $item->entity->name->value,
            $item->entity->field_css->value,
            $entity->id()
          );
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function serialize(Element $entity, array &$result = []) {
    $build = [
      'type' => 'structural',
      'id' => $entity->id(),
    ];
    $fields = $this->collectFields($entity, 'serialize');
    foreach ($fields as $placeholder => $value) {
      $fields[$placeholder] = $value;
    }
    $build['fields'] = $fields;
    $build['classes'] = $entity->field_classes->value;
    $styles = $entity->field_styles;
    foreach ($styles as $item) {
      $build['styles'][$item->entity->name->value] = $item->entity->field_css->value;
    }
    $result = $build + $result;
  }

  /**
   * {@inheritdoc}
   */
  public function describe(Element $entity, array &$result = []) {
    parent::describe($entity, $result);
    $result['fields'] = [];

    // Collect field description.
    $patternDefinition = $this->getPatternDefinition($entity->field_pattern->value);
    if ($patternDefinition != NULL) {
      foreach ($entity->children as $item) {
        if ($item->entity != NULL) {
          $fieldData = $this->elementHandler->describe($item->entity);
          $fieldData = $fieldData + $patternDefinition->getField($item->entity->field_placeholder->value)->toArray();
          $result['fields'][] = $fieldData;
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function collectFields(Element $entity, $function = 'get') {
    $fields = [];
    $pattern = $entity->field_pattern->value;
    $patternDefinition = $this->getPatternDefinition($pattern);
    if ($patternDefinition != NULL) {
      $numFields = count($patternDefinition->getFields());
      $i = 0;
      foreach ($entity->children as $item) {
        if ($item->entity != NULL && $item->entity->hasField('field_placeholder')) {
          if ($i >= $numFields) {
            break;
          }
          $placeholder = $item->entity->field_placeholder->value;
          if ($patternDefinition->hasField($placeholder)) {
            $item->entity->setHandlerType($patternDefinition->getField($placeholder)->getType());
            $fields[$placeholder] = $this->elementHandler->{$function}($item->entity);
          }
          $i++;
        }
      }
    }
    return $fields;
  }

  /**
   * Alter the markup to add attributes to rows and components.
   *
   * @param string $markup
   *   The markup to alter.
   * @param \Drupal\pagedesigner\Entity\Element $entity
   *   The entity to use.
   * @param bool $edit
   *   Whether to add the edit attributes.
   *
   * @return string
   *   The altered markup
   */
  public static function augmentEditPattern($markup, Element $entity, $edit = FALSE) {
    $dom = new \DOMDocument();
    libxml_use_internal_errors(TRUE);
    $dom->loadHTML(mb_convert_encoding($markup, 'HTML-ENTITIES', 'UTF-8'), LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD | LIBXML_NOWARNING);
    libxml_clear_errors();
    $root = $dom->documentElement;
    $root->normalize();
    $root->setAttribute('id', 'pd-cp-' . $entity->id());
    $classes = '';
    $patternClasses = explode(' ', $root->getAttribute('class'));
    $styleClasses = explode(' ', $entity->field_classes->value);
    $classes = implode(' ', array_unique(array_merge($patternClasses, $styleClasses)));
    $root->setAttribute('class', $classes);
    if ($edit) {
      $root->setAttribute('data-entity-id', $entity->id());
      $root->setAttribute('data-gjs-type', $entity->field_pattern->value);
    }
    $columns = [];
    $children = $root->childNodes;
    foreach ($children as $item) {
      if (strpos($item->nodeName, '#') !== 0 && strpos($item->getAttribute('class'), 'iq-column') !== FALSE) {
        $columns[] = $item;
      }
    }
    $i = 0;
    foreach ($entity->children as $item) {
      if ($item->entity != NULL && $item->entity->bundle() == 'cell') {
        if ($item->entity == NULL || empty($columns[$i])) {
          continue;
        }
        $column = $columns[$i];
        if (empty($column->getAttribute('id'))) {
          $column->setAttribute('id', 'pd-cp-' . $item->entity->id());
        }
        if ($edit) {
          if (empty($column->getAttribute('data-entity-id'))) {
            $column->setAttribute('data-entity-id', $item->entity->id());
          }
          if (empty($column->getAttribute('data-gjs-type'))) {
            $column->setAttribute('data-gjs-type', 'cell');
          }
        }
        $i++;
      }
    }
    return $dom->saveHTML();
  }

  /**
   * {@inheritdoc}
   */
  public function getContent(Element $entity, array &$list = []) {
    foreach ($entity->children as $item) {
      if ($item->entity != NULL && ($child = $item->entity->loadNewestPublished()) != NULL) {
        $this->elementHandler->getContent($child, $list);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function publish(Element $entity) {
    parent::publish($entity);
    foreach ($entity->field_styles as $style) {
      $style->entity->setPublished(TRUE);
      $style->entity->save();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function generate($definition, array $data, Element &$entity = NULL) {
    // Creating entity.
    parent::generate(['type' => $definition['type'], 'name' => $definition['id']], $data, $entity);
    $entity->field_pattern->value = $definition['id'];
    $entity->parent->target_id = $data['parent'];
    $entity->container->target_id = $data['container'];
    $entity->entity->target_id = $data['entity'];
    $entity->saveEdit();

    // Creating field entities.
    $fields = $definition['patternDefinition']->getFields();
    foreach ($fields as $key => $field) {
      // Create field info array.
      $fieldData = $field->toArray();
      $fieldData['parent'] = $entity->id();
      $fieldData['container'] = $data['container'];
      $fieldData['entity'] = $data['entity'];
      $fieldData['placeholder'] = $key;

      // Create field entity and add as child.
      $fieldElement = $this->elementHandler->generate(['type' => $field->getType()], $fieldData);
      $fieldElement->saveEdit();
      $entity->children->appendItem($fieldElement);
    }
    $entity->saveEdit();

    // Adding entity to parent cell if provided.
    $cell = $entity->parent->entity;
    if ($cell != NULL) {
      $cell = $cell->getTranslation($entity->langcode->value);
      $cell->children->appendItem($entity);
      $cell->saveEdit();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function copy(Element $entity, Element $container = NULL, Element &$clone = NULL) {
    parent::copy($entity, $container, $clone);
    $styles = [];
    foreach ($clone->field_styles as $style) {
      $styleCopy = $style->entity->createDuplicate();
      $styleCopy->parent->entity = $clone;
      $styleCopy->container->entity = $container;
      if ($container != NULL) {
        $styleCopy->entity->entity = $container->entity->entity;
      }
      else {
        $styleCopy->entity->entity = NULL;
      }
      $styleCopy->langcode->value = $clone->langcode->value;
      $styles[] = $styleCopy;
    }
    $clone->field_styles->setValue($styles);
    $clone->setNewRevision(FALSE);
    $clone->save();
    return $clone;
  }

  /**
   * {@inheritdoc}
   */
  public function patch(Element $entity, array $data) {
    if (!empty($data['fields'])) {
      // Updating fields.
      $pattern = $entity->field_pattern->value;
      $patternDefinition = $this->getPatternDefinition($pattern);
      if ($patternDefinition != NULL) {
        foreach ($entity->children as $item) {
          $placeholder = $item->entity->field_placeholder->value;
          if ($patternDefinition->hasField($placeholder)) {
            $field = $patternDefinition->getField($placeholder);
            $fieldData = $data['fields'][$placeholder];
            if (!\is_array($fieldData)) {
              $fieldData = [$fieldData];
            }
            $this->elementHandler->patch($item->entity, $fieldData);
            $storedFields[$placeholder] = 1;
          }
        }

        // Add missing fields.
        $patternFields = $patternDefinition->getFields();
        if (count($storedFields) < count($patternFields)) {
          foreach ($patternFields as $field) {
            if (empty($storedFields[$field->getName()])) {
              $fieldData = $field->toArray();
              $fieldData['parent'] = $entity->id();
              $fieldData['container'] = $entity->container->target_id;
              $fieldData['entity'] = $entity->entity->target_id;
              $fieldData['placeholder'] = $field->getName();
              // Get handler, create field entity and add as child.
              $fieldElement = $this->elementHandler->generate(['type' => $field->getType()], $fieldData);
              if (!empty($data['fields'][$field->getName()])) {
                $fieldData = $data['fields'][$placeholder];
                if (!\is_array($fieldData)) {
                  $fieldData = [$fieldData];
                }
                $this->elementHandler->patch($fieldElement, $fieldData);
              }
              $fieldElement->saveEdit();
              $entity->children->appendItem($fieldElement);
            }
          }
        }
      }
    }

    $styles = $entity->field_styles;
    $stylingPermission = \Drupal::currentUser()->hasPermission('edit pagedesigner styles');
    if (!empty($data['styles']) && $stylingPermission) {
      $saved = [];

      foreach ($styles as $item) {
        $saved[$item->entity->name->value] = $item->entity;
      }
      foreach ($data['styles'] as $key => $css) {
        $style = NULL;
        if (!empty($saved[$key])) {
          $style = $saved[$key];
        }
        else {
          $style = Element::create(['type' => 'style', 'name' => $key]);
          $entity->field_styles->appendItem($style);
          $style->langcode->value = $entity->langcode->value;
        }
        $style->field_css->value = $css;
        $style->parent->target_id = $entity->id();
        $style->container->target_id = $entity->container->target_id;
        $style->saveEdit();
      }
    }
    if (!empty($data['classes'])) {
      $entity->field_classes->value = implode(' ', $data['classes']);
    }
    $entity->saveEdit();
  }

  /**
   * {@inheritdoc}
   */
  public function delete(Element $entity, bool $remove = FALSE) {
    parent::delete($entity, $remove);
    foreach ($entity->field_styles as $item) {
      if ($remove) {
        $entity->delete();
      }
      else {
        $item->entity->deleted = TRUE;
        $item->entity->saveEdit();
      }
    }
  }

}
