<?php

namespace Drupal\pagedesigner\Service;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Render\Markup;
use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\PagedesignerService;
use Drupal\pagedesigner\Service\Render\Styles;
use Drupal\user\Entity\User;

/**
 * The renderer service allows to render a pagedesigner element container.
 */
class Renderer extends PagedesignerService {

  /**
   * Style object to collect the styles during rendering.
   *
   * @var Drupal\pagedesigner\Service\Render\Styles|null
   */
  protected static $styles = NULL;

  /**
   * Add a style definition.
   *
   * @param string $key
   *   The size for which to add the style definition (mn, sm, xs).
   * @param string $style
   *   The style definition to add.
   * @param int $id
   *   The id of the element.
   */
  public static function addStyle($key, $style, $id) {
    if (self::$styles == NULL) {
      self::$styles = new Styles();
    }
    self::$styles->addStyle($key, $style, $id);
  }

  /**
   * Render a container for a certain entity.
   *
   * @param \Drupal\pagedesigner\Entity\Element $container
   *   The container to render.
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *   The entity the container is rendered for.
   *
   * @return \Drupal\pagedesigner\Service\Renderer
   *   The renderer service.
   */
  public function render(Element $container, ContentEntityBase $entity = NULL) {
    if ($container == NULL) {
      return $this;
    }
    $this->_output = $this->getHandler($container)->render($container);
    if (empty($this->_output['#attached'])) {
      $this->_output['#attached'] = [];
    }
    $user = \Drupal::currentUser();
    if ($user->hasPermission('edit pagedesigner element entities') && $entity->access('update', $user)) {
      $this->_output['#attached']['library'][] = 'pagedesigner/icon';
    }
    $this->_output['#cache']['max-age'] = 0;
    $this->addStyles();
    return $this;
  }

  /**
   * Publicly render a container for a certain entity.
   *
   * @param \Drupal\pagedesigner\Entity\Element $container
   *   The container to render.
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *   The entity the container is rendered for.
   *
   * @return \Drupal\pagedesigner\Service\Renderer
   *   The renderer service.
   */
  public function renderForPublic(Element $container, ContentEntityBase $entity = NULL) {
    if ($container == NULL) {
      return $this;
    }
    $this->_output = $this->getHandler($container)->renderForPublic($container);
    $this->addStyles();
    return $this;
  }

  /**
   * Editebly render a container for a certain entity.
   *
   * @param \Drupal\pagedesigner\Entity\Element $container
   *   The container to render.
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *   The entity the container is rendered for.
   *
   * @return \Drupal\pagedesigner\Service\Renderer
   *   The renderer service.
   */
  public function renderForEdit(Element $container, ContentEntityBase $entity = NULL) {
    if ($container == NULL) {
      return $this;
    }

    if ($entity != NULL) {
      $locker = \Drupal::service('pagedesigner.locker');
      $locker->setEntity($entity);

      if (!empty(\Drupal::request()->query->get('otherTab'))) {
        $username = 'unknown';
        $otherUser = User::load($locker->getMetaData()->owner);
        if ($otherUser != NULL) {
          $username = $otherUser->getUsername();
        }
        drupal_set_message(t('This page is currently being edited by you in another tab. Continue editing in that tab or close the other tab and reload here.', []), 'warning');
        return $this->render($container, $entity);
      }
      $lock = $locker->acquire();
      if (!$lock && !$locker->hasLock()) {
        $username = 'unknown';
        $otherUser = User::load($locker->getMetaData()->owner);
        if ($otherUser != NULL) {
          $username = $otherUser->getUsername();
        }
        drupal_set_message(t('This page is currently being edited by user %user. You may access it again after the user is done editing.', ['%user' => $username]), 'warning');
        return $this->render($container, $entity);
      }
    }

    $this->_output = $this->getHandler($container)->renderForEdit($container);
    $this->_output['#cache']['max-age'] = 0;
    $this->addStyles();
    $this->addEditorAttachments();
    $this->addPagedesigner();
    $this->addPatternAttachments();
    return $this;
  }

  /**
   * Return the markup generated by the last render.
   *
   * @return string
   *   The generated markup
   */
  public function getMarkup() {
    return Markup::create(\Drupal::service('renderer')->render($this->_output));
  }

  /**
   * Return the styles generated by the last render.
   *
   * @return string
   *   The generated styles.
   */
  public function getStyles() {
    $rendered = '';
    if (self::$styles != NULL) {
      $output = [
        'large' => '',
        'medium' => '@media (max-width: 992px)',
        'small' => '@media (max-width: 768px)',
      ];
      foreach ($output as $size => $query) {
        if (empty($query)) {
          $rendered .= self::$styles->getStyles($size) . "\n";
        }
        else {
          $rendered .= $query . '{' . self::$styles->getStyles($size) . '}' . "\n";
        }
      }
    }
    return $rendered;
  }

  /**
   * Add the collected styles of the elements to the output.
   */
  protected function addStyles() {
    if (self::$styles != NULL) {
      $rendered = '';
      $output = [
        'large' => '',
        'medium' => '@media (max-width: 992px)',
        'small' => '@media (max-width: 768px)',
      ];
      foreach ($output as $size => $query) {
        if (empty($query)) {
          $rendered .= self::$styles->getStyles($size) . "\n";
        }
        else {
          $rendered .= $query . '{' . self::$styles->getStyles($size) . '}' . "\n";
        }
      }
      $this->_output['#attached']['html_head']['pd_styles'] =
      [
            [
              '#tag' => 'style',
              '#value' => $rendered,
              '#attributes' => [
                'id' => 'pd_styles',
              ],
            ], 'pagedesigner_dynamic_css',
      ];
    }
  }

  /**
   * Add attachments for the WYSIWYG editor to the output.
   */
  protected function addEditorAttachments() {
    if (empty($this->_output['#attached'])) {
      $this->_output['#attached'] = [];
    }
    $formats = ['pagedesigner'];
    $this->_output['#attached'] = array_merge(
        $this->_output['#attached'],
        \Drupal::service('plugin.manager.editor')->getAttachments($formats)
    );
  }

  /**
   * Attach pagedesigner library.
   *
   * Adds the basic pagedesigner edit and pattern libraries needed to
   * edit to the pagedesigner to the output.
   */
  protected function addPagedesigner() {
    if (empty($this->_output['#attached'])) {
      $this->_output['#attached'] = [];
    }
    $this->_output['#attached']['library'][] = 'pagedesigner/pagedesigner';
    $this->elementHandler->collectAttachments($this->_output['#attached']);
  }

  /**
   * Adds the attachments of all pattens to the output.
   */
  protected function addPatternAttachments() {
    if (empty($this->_output['#attached'])) {
      $this->_output['#attached'] = [];
    }

    $patternDefinitions = \Drupal::service('plugin.manager.ui_patterns')->getDefinitions();
    $this->elementHandler->collectPatterns($patternDefinitions);

    foreach ($patternDefinitions as $definition) {
      if ($this->elementHandler->isInTheme($definition)) {
        foreach ($definition->getLibrariesNames() as $library) {
          $this->_output['#attached']['library'][] = $library;
        }
      }
    }
  }

}
